# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_sqlite_instance_in_process_env] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_sqlite_instance_out_of_process_env] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_sqlite_instance_multi_location] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_in_memory_instance_in_process_env] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_in_memory_instance_out_of_process_env] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_in_memory_instance_multi_location] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_in_memory_instance_grpc] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_sqlite_instance_deployed_grpc] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_sqlite_instance_grpc] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_postgres_instance_in_process_env] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_postgres_instance_out_of_process_env] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_postgres_instance_multi_location] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_postgres_instance_grpc] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_sqlite_instance_managed_grpc_env] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_sqlite_instance_deployed_grpc_env] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}

snapshots['TestPresets.test_basic_preset_query_with_presets[readonly_in_memory_instance_managed_grpc_env] 1'] = {
    'pipelineOrError': {
        'name': 'csv_hello_world',
        'presets': [
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'prod',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num_prod.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: data/num.csv
''',
                'solidSelection': None
            },
            {
                '__typename': 'PipelinePreset',
                'mode': 'default',
                'name': 'test_inline',
                'runConfigYaml': '''solids:
  sum_solid:
    inputs:
      num: /data/num.csv
''',
                'solidSelection': None
            }
        ]
    }
}



from dagster.core.test_utils import instance_for_test
from dagster.serdes import serialize_pp

from .setup import get_main_external_repo


def test_all_snapshot_ids(snapshot):
    # This ensures that pipeline snapshots remain stable
    # If you 1) change any pipelines in dagster_graphql_test or 2) change the
    # schema of PipelineSnapshots you are free to rerecord
    with instance_for_test() as instance:
        repo = get_main_external_repo(instance)
        for pipeline in sorted(repo.get_all_external_pipelines(), key=lambda p: p.name):
            snapshot.assert_match(serialize_pp(pipeline.pipeline_snapshot))
            snapshot.assert_match(pipeline.computed_pipeline_snapshot_id)



import yaml
from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.core.host_representation import (
    ExternalSchedule,
    ExternalScheduleExecutionData,
    PipelineSelector,
    RepositorySelector,
    ScheduleSelector,
)

from .utils import UserFacingGraphQLError, capture_dauphin_error


@capture_dauphin_error
def reconcile_scheduler_state(graphene_info, repository_selector):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(repository_selector, "repository_selector", RepositorySelector)

    location = graphene_info.context.get_repository_location(repository_selector.location_name)
    repository = location.get_repository(repository_selector.repository_name)
    instance = graphene_info.context.instance

    instance.reconcile_scheduler_state(repository)
    return graphene_info.schema.type_named("ReconcileSchedulerStateSuccess")(message="Success")


@capture_dauphin_error
def start_schedule(graphene_info, schedule_selector):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(schedule_selector, "schedule_selector", ScheduleSelector)
    location = graphene_info.context.get_repository_location(schedule_selector.location_name)
    repository = location.get_repository(schedule_selector.repository_name)
    instance = graphene_info.context.instance
    schedule_state = instance.start_schedule_and_update_storage_state(
        repository.get_external_schedule(schedule_selector.schedule_name)
    )
    return graphene_info.schema.type_named("ScheduleStateResult")(
        schedule_state=graphene_info.schema.type_named("ScheduleState")(
            graphene_info, schedule_state=schedule_state
        )
    )


@capture_dauphin_error
def stop_schedule(graphene_info, schedule_origin_id):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    instance = graphene_info.context.instance
    schedule_state = instance.stop_schedule_and_update_storage_state(schedule_origin_id)
    return graphene_info.schema.type_named("ScheduleStateResult")(
        schedule_state=graphene_info.schema.type_named("ScheduleState")(
            graphene_info, schedule_state=schedule_state
        )
    )


@capture_dauphin_error
def get_scheduler_or_error(graphene_info):
    instance = graphene_info.context.instance

    if not instance.scheduler:
        raise UserFacingGraphQLError(graphene_info.schema.type_named("SchedulerNotDefinedError")())

    return graphene_info.schema.type_named("Scheduler")(
        scheduler_class=instance.scheduler.__class__.__name__
    )


@capture_dauphin_error
def get_schedule_states_or_error(
    graphene_info, repository_selector, with_no_schedule_definition_filter=None
):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.opt_inst_param(repository_selector, "repository_selector", RepositorySelector)
    check.opt_bool_param(
        with_no_schedule_definition_filter, "with_no_schedule_definition_filter", default=False
    )

    instance = graphene_info.context.instance
    if not repository_selector:
        # Return all schedule states
        results = [
            graphene_info.schema.type_named("ScheduleState")(
                graphene_info, schedule_state=schedule_state
            )
            for schedule_state in instance.all_stored_schedule_state()
        ]
        return graphene_info.schema.type_named("ScheduleStates")(results=results)

    location = graphene_info.context.get_repository_location(repository_selector.location_name)
    repository = location.get_repository(repository_selector.repository_name)
    repository_origin_id = repository.get_origin().get_id()
    instance = graphene_info.context.instance

    results = [
        graphene_info.schema.type_named("ScheduleState")(
            graphene_info, schedule_state=schedule_state
        )
        for schedule_state in instance.all_stored_schedule_state(
            repository_origin_id=repository_origin_id
        )
    ]

    if with_no_schedule_definition_filter:
        external_schedules = repository.get_external_schedules()
        external_schedule_origin_ids = set(
            external_schedule.get_origin_id() for external_schedule in external_schedules
        )

        # Filter for all schedule states for which there are no matching external schedules with the
        # same origin id
        results = list(
            filter(
                lambda schedule_state: schedule_state.schedule_origin_id
                not in external_schedule_origin_ids,
                results,
            )
        )

    return graphene_info.schema.type_named("ScheduleStates")(results=results)


@capture_dauphin_error
def get_schedule_definitions_or_error(graphene_info, repository_selector):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(repository_selector, "repository_selector", RepositorySelector)

    location = graphene_info.context.get_repository_location(repository_selector.location_name)
    repository = location.get_repository(repository_selector.repository_name)
    external_schedules = repository.get_external_schedules()

    results = [
        graphene_info.schema.type_named("ScheduleDefinition")(
            graphene_info, external_schedule=external_schedule
        )
        for external_schedule in external_schedules
    ]

    return graphene_info.schema.type_named("ScheduleDefinitions")(results=results)


def get_schedule_definitions_for_pipeline(graphene_info, pipeline_selector):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(pipeline_selector, "pipeline_selector", PipelineSelector)

    location = graphene_info.context.get_repository_location(pipeline_selector.location_name)
    repository = location.get_repository(pipeline_selector.repository_name)
    external_schedules = repository.get_external_schedules()

    return [
        graphene_info.schema.type_named("ScheduleDefinition")(
            graphene_info, external_schedule=external_schedule
        )
        for external_schedule in external_schedules
        if external_schedule.pipeline_name == pipeline_selector.pipeline_name
    ]


@capture_dauphin_error
def get_schedule_definition_or_error(graphene_info, schedule_selector):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(schedule_selector, "schedule_selector", ScheduleSelector)
    location = graphene_info.context.get_repository_location(schedule_selector.location_name)
    repository = location.get_repository(schedule_selector.repository_name)

    external_schedule = repository.get_external_schedule(schedule_selector.schedule_name)
    if not external_schedule:
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named("ScheduleDefinitionNotFoundError")(
                schedule_name=schedule_selector.schedule_name
            )
        )

    return graphene_info.schema.type_named("ScheduleDefinition")(
        graphene_info, external_schedule=external_schedule
    )


def get_schedule_config(graphene_info, external_schedule):
    check.inst_param(external_schedule, "external_schedule", ExternalSchedule)
    handle = external_schedule.handle.repository_handle

    result = graphene_info.context.get_external_schedule_execution_data(
        handle, external_schedule.name
    )
    if isinstance(result, ExternalScheduleExecutionData):
        run_config_yaml = yaml.safe_dump(result.run_config, default_flow_style=False)
        return graphene_info.schema.type_named("ScheduleRunConfig")(
            yaml=run_config_yaml if run_config_yaml else ""
        )
    else:
        return graphene_info.schema.type_named("PythonError")(result.error)



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['test_mode_fetch_resources 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'modes': [
            {
                'name': 'add_mode',
                'resources': [
                    {
                        'configField': {
                            'configType': {
                                'key': 'Int'
                            }
                        },
                        'description': None,
                        'name': 'op'
                    }
                ]
            },
            {
                'name': 'double_adder',
                'resources': [
                    {
                        'configField': {
                            'configType': {
                                'fields': [
                                    {
                                        'configType': {
                                            'key': 'Int'
                                        },
                                        'name': 'num_one'
                                    },
                                    {
                                        'configType': {
                                            'key': 'Int'
                                        },
                                        'name': 'num_two'
                                    }
                                ],
                                'key': 'Shape.fc3adbbf54d7ee8b03e7f0116e13d34e253c5bcf'
                            }
                        },
                        'description': None,
                        'name': 'op'
                    }
                ]
            },
            {
                'name': 'mult_mode',
                'resources': [
                    {
                        'configField': {
                            'configType': {
                                'key': 'Int'
                            }
                        },
                        'description': None,
                        'name': 'op'
                    }
                ]
            }
        ]
    }
}

snapshots['test_required_resources 1'] = {
    'pipelineOrError': {
        'name': 'required_resource_pipeline',
        'solids': [
            {
                'definition': {
                    'requiredResources': [
                        {
                            'resourceKey': 'R1'
                        }
                    ]
                }
            }
        ]
    }
}



from __future__ import absolute_import

from dagster_graphql import dauphin
from dagster_graphql.implementation.fetch_solids import get_solid, get_solids

from dagster import check
from dagster.core.code_pointer import (
    CodePointer,
    FileCodePointer,
    ModuleCodePointer,
    PackageCodePointer,
)
from dagster.core.host_representation import (
    ExternalRepository,
    PythonEnvRepositoryLocationHandle,
    RepositoryLocation,
)
from dagster.core.origin import RepositoryGrpcServerOrigin, RepositoryPythonOrigin


class DauphinRepository(dauphin.ObjectType):
    class Meta(object):
        name = "Repository"

    def __init__(self, repository, repository_location):
        self._repository = check.inst_param(repository, "repository", ExternalRepository)
        self._repository_location = check.inst_param(
            repository_location, "repository_location", RepositoryLocation
        )
        super(DauphinRepository, self).__init__(name=repository.name)

    id = dauphin.NonNull(dauphin.ID)
    name = dauphin.NonNull(dauphin.String)
    location = dauphin.NonNull("RepositoryLocation")
    pipelines = dauphin.non_null_list("Pipeline")
    usedSolids = dauphin.Field(dauphin.non_null_list("UsedSolid"))
    usedSolid = dauphin.Field("UsedSolid", name=dauphin.NonNull(dauphin.String))
    origin = dauphin.NonNull("RepositoryOrigin")
    partitionSets = dauphin.non_null_list("PartitionSet")

    def resolve_id(self, _graphene_info):
        return self._repository.get_origin_id()

    def resolve_origin(self, graphene_info):
        origin = self._repository.get_origin()
        if isinstance(origin, RepositoryGrpcServerOrigin):
            return graphene_info.schema.type_named("GrpcRepositoryOrigin")(origin)
        else:
            return graphene_info.schema.type_named("PythonRepositoryOrigin")(origin)

    def resolve_location(self, graphene_info):
        return graphene_info.schema.type_named("RepositoryLocation")(self._repository_location)

    def resolve_pipelines(self, graphene_info):
        return sorted(
            [
                graphene_info.schema.type_named("Pipeline")(pipeline)
                for pipeline in self._repository.get_all_external_pipelines()
            ],
            key=lambda pipeline: pipeline.name,
        )

    def resolve_usedSolid(self, _graphene_info, name):
        return get_solid(self._repository, name)

    def resolve_usedSolids(self, _graphene_info):
        return get_solids(self._repository)

    def resolve_partitionSets(self, graphene_info):
        return (
            graphene_info.schema.type_named("PartitionSet")(self._repository.handle, partition_set)
            for partition_set in self._repository.get_external_partition_sets()
        )


class DauphinPythonRepositoryOrigin(dauphin.ObjectType):
    class Meta(object):
        name = "PythonRepositoryOrigin"

    executable_path = dauphin.NonNull(dauphin.String)
    code_pointer = dauphin.NonNull("CodePointer")

    def __init__(self, origin):
        self._origin = check.inst_param(origin, "origin", RepositoryPythonOrigin)

    def resolve_executable_path(self, _graphene_info):
        return self._origin.executable_path

    def resolve_code_pointer(self, graphene_info):
        return graphene_info.schema.type_named("CodePointer")(self._origin.code_pointer)


class DauphinGrpcRepositoryOrigin(dauphin.ObjectType):
    class Meta(object):
        name = "GrpcRepositoryOrigin"

    grpc_url = dauphin.NonNull(dauphin.String)

    def __init__(self, origin):
        self._origin = check.inst_param(origin, "origin", RepositoryGrpcServerOrigin)

    def resolve_grpc_url(self, _graphene_info):
        return "grpc:{host}:{socket_or_port}".format(
            host=self._origin.host,
            socket_or_port=(self._origin.socket if self._origin.socket else self._origin.port),
        )


class DauphinRepositoryOrigin(dauphin.Union):
    class Meta(object):
        name = "RepositoryOrigin"
        types = ("PythonRepositoryOrigin", "GrpcRepositoryOrigin")


class DauphinRepositoryLocation(dauphin.ObjectType):
    class Meta(object):
        name = "RepositoryLocation"

    name = dauphin.NonNull(dauphin.String)
    is_reload_supported = dauphin.NonNull(dauphin.Boolean)
    environment_path = dauphin.String()
    repositories = dauphin.non_null_list("Repository")

    def __init__(self, location):
        self._location = check.inst_param(location, "location", RepositoryLocation)
        environment_path = (
            location.location_handle.executable_path
            if isinstance(location.location_handle, PythonEnvRepositoryLocationHandle)
            else None
        )

        check.invariant(location.name is not None)

        super(DauphinRepositoryLocation, self).__init__(
            name=location.name,
            environment_path=environment_path,
            is_reload_supported=location.is_reload_supported,
        )

    def resolve_repositories(self, graphene_info):
        return [
            graphene_info.schema.type_named("Repository")(repository, self._location)
            for repository in self._location.get_repositories().values()
        ]


class DauphinCodePointer(dauphin.ObjectType):
    class Meta(object):
        name = "CodePointer"

    description = dauphin.NonNull(dauphin.String)
    metadata = dauphin.non_null_list("CodePointerMetadata")

    def __init__(self, code_pointer):
        self._code_pointer = check.inst_param(code_pointer, "code_pointer", CodePointer)

    def resolve_description(self, _graphene_info):
        return self._code_pointer.describe()

    def resolve_metadata(self, graphene_info):
        metadata = {}
        if isinstance(self._code_pointer, FileCodePointer):
            metadata["python_file"] = self._code_pointer.python_file
            metadata["attribute"] = self._code_pointer.fn_name
        if isinstance(self._code_pointer, ModuleCodePointer):
            metadata["python_module"] = self._code_pointer.module
            metadata["attribute"] = self._code_pointer.fn_name
        if isinstance(self._code_pointer, PackageCodePointer):
            metadata["python_package"] = self._code_pointer.module
            metadata["attribute"] = self._code_pointer.attribute

        return [
            graphene_info.schema.type_named("CodePointerMetadata")(key=key, value=value)
            for key, value in metadata.items()
        ]


class DauphinCodePointerMetadata(dauphin.ObjectType):
    class Meta(object):
        name = "CodePointerMetadata"

    key = dauphin.NonNull(dauphin.String)
    value = dauphin.NonNull(dauphin.String)


class DauphinRepositoryConnection(dauphin.ObjectType):
    class Meta(object):
        name = "RepositoryConnection"

    nodes = dauphin.non_null_list("Repository")



from graphql.execution.base import ResolveInfo

from dagster import PipelineDefinition, check
from dagster.config.validate import validate_config
from dagster.core.definitions import create_run_config_schema
from dagster.core.host_representation import PipelineSelector
from dagster.core.storage.pipeline_run import PipelineRunsFilter
from dagster.core.storage.tags import TagType, get_tag_type

from .external import ensure_valid_config, get_external_pipeline_or_raise
from .utils import UserFacingGraphQLError, capture_dauphin_error


def is_config_valid(pipeline_def, run_config, mode):
    check.str_param(mode, "mode")
    check.inst_param(pipeline_def, "pipeline_def", PipelineDefinition)

    run_config_schema = create_run_config_schema(pipeline_def, mode)
    validated_config = validate_config(run_config_schema.environment_type, run_config)
    return validated_config.success


def get_validated_config(pipeline_def, run_config, mode):
    from dagster_graphql.schema.errors import DauphinPipelineConfigValidationInvalid

    check.str_param(mode, "mode")
    check.inst_param(pipeline_def, "pipeline_def", PipelineDefinition)

    run_config_schema = create_run_config_schema(pipeline_def, mode)

    validated_config = validate_config(run_config_schema.environment_type, run_config)

    if not validated_config.success:
        raise UserFacingGraphQLError(
            DauphinPipelineConfigValidationInvalid.for_validation_errors(
                pipeline_def.get_external_pipeline(), validated_config.errors
            )
        )

    return validated_config


def get_run_by_id(graphene_info, run_id):
    instance = graphene_info.context.instance
    run = instance.get_run_by_id(run_id)
    if not run:
        return graphene_info.schema.type_named("PipelineRunNotFoundError")(run_id)
    else:
        return graphene_info.schema.type_named("PipelineRun")(run)


def get_run_tags(graphene_info):
    instance = graphene_info.context.instance
    return [
        graphene_info.schema.type_named("PipelineTagAndValues")(key=key, values=values)
        for key, values in instance.get_run_tags()
        if get_tag_type(key) != TagType.HIDDEN
    ]


@capture_dauphin_error
def get_run_group(graphene_info, run_id):
    instance = graphene_info.context.instance
    result = instance.get_run_group(run_id)

    if result is None:
        return graphene_info.schema.type_named("RunGroupNotFoundError")(run_id)
    else:
        root_run_id, run_group = result
        return graphene_info.schema.type_named("RunGroup")(
            root_run_id=root_run_id,
            runs=[graphene_info.schema.type_named("PipelineRun")(run) for run in run_group],
        )


def get_runs(graphene_info, filters, cursor=None, limit=None):
    check.opt_inst_param(filters, "filters", PipelineRunsFilter)
    check.opt_str_param(cursor, "cursor")
    check.opt_int_param(limit, "limit")

    instance = graphene_info.context.instance
    runs = []

    if filters and filters.run_ids and len(filters.run_ids) == 1:
        run = instance.get_run_by_id(filters.run_ids[0])
        if run:
            runs = [run]
    elif filters and (
        filters.pipeline_name or filters.tags or filters.status or filters.snapshot_id
    ):
        runs = instance.get_runs(filters, cursor, limit)
    else:
        runs = instance.get_runs(cursor=cursor, limit=limit)

    return [graphene_info.schema.type_named("PipelineRun")(run) for run in runs]


def get_run_groups(graphene_info, filters=None, cursor=None, limit=None):
    from ..schema.runs import DauphinRunGroup

    check.opt_inst_param(filters, "filters", PipelineRunsFilter)
    check.opt_str_param(cursor, "cursor")
    check.opt_int_param(limit, "limit")

    instance = graphene_info.context.instance
    run_groups = instance.get_run_groups(filters=filters, cursor=cursor, limit=limit)

    for root_run_id in run_groups:
        run_groups[root_run_id]["runs"] = [
            graphene_info.schema.type_named("PipelineRun")(run)
            for run in run_groups[root_run_id]["runs"]
        ]

    return [
        DauphinRunGroup(root_run_id=root_run_id, runs=run_group["runs"])
        for root_run_id, run_group in run_groups.items()
    ]


@capture_dauphin_error
def validate_pipeline_config(graphene_info, selector, run_config, mode):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(selector, "selector", PipelineSelector)
    check.opt_str_param(mode, "mode")

    external_pipeline = get_external_pipeline_or_raise(graphene_info, selector)
    ensure_valid_config(external_pipeline, mode, run_config)
    return graphene_info.schema.type_named("PipelineConfigValidationValid")(
        pipeline_name=external_pipeline.name
    )


@capture_dauphin_error
def get_execution_plan(graphene_info, selector, run_config, mode):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(selector, "selector", PipelineSelector)
    check.opt_str_param(mode, "mode")

    external_pipeline = get_external_pipeline_or_raise(graphene_info, selector)
    ensure_valid_config(external_pipeline, mode, run_config)
    return graphene_info.schema.type_named("ExecutionPlan")(
        graphene_info.context.get_external_execution_plan(
            external_pipeline=external_pipeline,
            mode=mode,
            run_config=run_config,
            step_keys_to_execute=None,
        )
    )


@capture_dauphin_error
def get_stats(graphene_info, run_id):
    stats = graphene_info.context.instance.get_run_stats(run_id)
    return graphene_info.schema.type_named("PipelineRunStatsSnapshot")(stats)


def get_step_stats(graphene_info, run_id):
    step_stats = graphene_info.context.instance.get_run_step_stats(run_id)
    return [graphene_info.schema.type_named("PipelineRunStepStats")(stats) for stats in step_stats]



from __future__ import absolute_import

from dagster_graphql import dauphin

from dagster import check
from dagster.core.host_representation import ExternalExecutionPlan
from dagster.core.snap import (
    ExecutionStepInputSnap,
    ExecutionStepOutputSnap,
    ExecutionStepSnap,
    PipelineSnapshot,
)

from .dagster_types import to_dauphin_dagster_type


class DauphinExecutionPlan(dauphin.ObjectType):
    class Meta(object):
        name = "ExecutionPlan"

    steps = dauphin.non_null_list("ExecutionStep")
    artifactsPersisted = dauphin.NonNull(dauphin.Boolean)

    def __init__(self, external_execution_plan):
        super(DauphinExecutionPlan, self).__init__()
        self._external_execution_plan = check.inst_param(
            external_execution_plan, external_execution_plan, ExternalExecutionPlan
        )

    def resolve_steps(self, _graphene_info):
        return [
            DauphinExecutionStep(
                self._external_execution_plan,
                self._external_execution_plan.get_step_by_key(step.key),
            )
            for step in self._external_execution_plan.get_steps_in_plan()
        ]

    def resolve_artifactsPersisted(self, _graphene_info):
        return self._external_execution_plan.execution_plan_snapshot.artifacts_persisted


class DauphinExecutionStepOutput(dauphin.ObjectType):
    class Meta(object):
        name = "ExecutionStepOutput"

    name = dauphin.NonNull(dauphin.String)
    type = dauphin.Field(dauphin.NonNull("DagsterType"))

    def __init__(self, pipeline_snapshot, step_output_snap):
        super(DauphinExecutionStepOutput, self).__init__()
        self._step_output_snap = check.inst_param(
            step_output_snap, "step_output_snap", ExecutionStepOutputSnap
        )
        self._pipeline_snapshot = check.inst_param(
            pipeline_snapshot, "pipeline_snapshot", PipelineSnapshot
        )

    def resolve_name(self, _graphene_info):
        return self._step_output_snap.name

    def resolve_type(self, _graphene_info):
        return to_dauphin_dagster_type(
            self._pipeline_snapshot, self._step_output_snap.dagster_type_key
        )


class DauphinExecutionStepInput(dauphin.ObjectType):
    class Meta(object):
        name = "ExecutionStepInput"

    name = dauphin.NonNull(dauphin.String)
    type = dauphin.Field(dauphin.NonNull("DagsterType"))
    dependsOn = dauphin.non_null_list("ExecutionStep")

    def __init__(self, pipeline_snapshot, step_input_snap, external_execution_plan):
        super(DauphinExecutionStepInput, self).__init__()
        self._step_input_snap = check.inst_param(
            step_input_snap, "step_input_snap", ExecutionStepInputSnap
        )
        self._pipeline_snapshot = check.inst_param(
            pipeline_snapshot, "pipeline_snapshot", PipelineSnapshot
        )
        self._external_execution_plan = check.inst_param(
            external_execution_plan, "external_execution_plan", ExternalExecutionPlan
        )

    def resolve_name(self, _graphene_info):
        return self._step_input_snap.name

    def resolve_type(self, _graphene_info):
        return to_dauphin_dagster_type(
            self._pipeline_snapshot, self._step_input_snap.dagster_type_key
        )

    def resolve_dependsOn(self, graphene_info):
        return [
            graphene_info.schema.type_named("ExecutionStep")(
                self._external_execution_plan, self._external_execution_plan.get_step_by_key(key),
            )
            # We filter at this layer to ensure that we do not return outputs that
            # do not exist in the execution plan
            for key in filter(
                self._external_execution_plan.key_in_plan, self._step_input_snap.upstream_step_keys,
            )
        ]


class DauphinStepKind(dauphin.Enum):
    class Meta(object):
        name = "StepKind"

    COMPUTE = "COMPUTE"

    @property
    def description(self):
        # self ends up being the internal class "EnumMeta" in dauphin
        # so we can't do a dictionary lookup which is awesome
        if self == DauphinStepKind.COMPUTE:
            return "This is the user-defined computation step"
        else:
            return None


class DauphinExecutionStep(dauphin.ObjectType):
    class Meta(object):
        name = "ExecutionStep"

    key = dauphin.NonNull(dauphin.String)
    inputs = dauphin.non_null_list("ExecutionStepInput")
    outputs = dauphin.non_null_list("ExecutionStepOutput")
    solidHandleID = dauphin.NonNull(dauphin.String)
    kind = dauphin.NonNull("StepKind")
    metadata = dauphin.non_null_list("MetadataItemDefinition")

    def __init__(self, external_execution_plan, execution_step_snap):
        super(DauphinExecutionStep, self).__init__()
        self._external_execution_plan = check.inst_param(
            external_execution_plan, "external_execution_plan", ExternalExecutionPlan
        )
        self._plan_snapshot = external_execution_plan.execution_plan_snapshot
        self._step_snap = check.inst_param(
            execution_step_snap, "execution_step_snap", ExecutionStepSnap
        )

    def resolve_metadata(self, graphene_info):
        return [
            graphene_info.schema.type_named("MetadataItemDefinition")(key=mdi.key, value=mdi.value)
            for mdi in self._step_snap.metadata_items
        ]

    def resolve_inputs(self, graphene_info):
        return [
            graphene_info.schema.type_named("ExecutionStepInput")(
                self._external_execution_plan.represented_pipeline.pipeline_snapshot,
                inp,
                self._external_execution_plan,
            )
            for inp in self._step_snap.inputs
        ]

    def resolve_outputs(self, graphene_info):
        return [
            graphene_info.schema.type_named("ExecutionStepOutput")(
                self._external_execution_plan.represented_pipeline.pipeline_snapshot, out,
            )
            for out in self._step_snap.outputs
        ]

    def resolve_key(self, _graphene_info):
        return self._step_snap.key

    def resolve_solidHandleID(self, _graphene_info):
        return self._step_snap.solid_handle_id

    def resolve_kind(self, _graphene_info):
        return self._step_snap.kind



import copy

from dagster_graphql.test.utils import (
    define_out_of_process_context,
    execute_dagster_graphql,
    infer_pipeline_selector,
)
from dagster_graphql_tests.graphql.graphql_context_test_suite import (
    ExecutingGraphQLContextTestMatrix,
)

from dagster import execute_pipeline, lambda_solid, pipeline, repository, seven
from dagster.core.definitions.pipeline_base import InMemoryPipeline
from dagster.core.execution.api import execute_run
from dagster.core.instance import DagsterInstance
from dagster.core.storage.tags import PARENT_RUN_ID_TAG, ROOT_RUN_ID_TAG

RUNS_QUERY = """
query PipelineRunsRootQuery($selector: PipelineSelector!) {
  pipelineOrError(params: $selector) {
    ... on Pipeline {
      name
      pipelineSnapshotId
      runs {
        ...RunHistoryRunFragment
      }
    }
  }
}

fragment RunHistoryRunFragment on PipelineRun {
  runId
  status
  pipeline {
    ...on PipelineReference { name }
  }
  executionPlan {
    steps {
      key
    }
  }
  runConfigYaml
  mode
  canTerminate
  tags {
    key
    value
  }
}
"""

DELETE_RUN_MUTATION = """
mutation DeleteRun($runId: String!) {
  deletePipelineRun(runId: $runId) {
    __typename
    ... on DeletePipelineRunSuccess {
      runId
    }
    ... on PythonError {
      message
    }
  }
}
"""

ALL_TAGS_QUERY = """
{
  pipelineRunTags {
    ... on PipelineTagAndValues {
      key
      values
    }
  }
}
"""

ALL_RUNS_QUERY = """
{
  pipelineRunsOrError {
    ... on PipelineRuns {
      results {
        runId
        pipelineSnapshotId
        pipeline {
          __typename
          ... on PipelineReference {
            name
            solidSelection
          }
        }
      }
    }
  }
}
"""


FILTERED_RUN_QUERY = """
query PipelineRunsRootQuery($filter: PipelineRunsFilter!) {
  pipelineRunsOrError(filter: $filter) {
    ... on PipelineRuns {
      results {
        runId
      }
    }
  }
}
"""


RUN_GROUP_QUERY = """
query RunGroupQuery($runId: ID!) {
  runGroupOrError(runId: $runId) {
    ... on RunGroup {
      __typename
      rootRunId
      runs {
        runId
        pipeline {
          __typename
          ... on PipelineReference {
            name
          }
        }
      }
    }
    ... on RunGroupNotFoundError {
      __typename
      runId
      message
    }
  }
}
"""


ALL_RUN_GROUPS_QUERY = """
{
  runGroupsOrError {
    results {
      rootRunId
      runs {
        runId
        pipelineSnapshotId
        pipeline {
          __typename
          ... on PipelineReference {
            name
            solidSelection
          }
        }
      }
    }
  }
}
"""


def _get_runs_data(result, run_id):
    for run_data in result.data["pipelineOrError"]["runs"]:
        if run_data["runId"] == run_id:
            # so caller can delete keys
            return copy.deepcopy(run_data)

    return None


class TestGetRuns(ExecutingGraphQLContextTestMatrix):
    def test_get_runs_over_graphql(self, graphql_context):
        # This include needs to be here because its inclusion screws up
        # other code in this file which reads itself to load a repo
        from .utils import sync_execute_get_run_log_data

        selector = infer_pipeline_selector(graphql_context, "multi_mode_with_resources")

        payload_one = sync_execute_get_run_log_data(
            context=graphql_context,
            variables={
                "executionParams": {
                    "selector": selector,
                    "mode": "add_mode",
                    "runConfigData": {"resources": {"op": {"config": 2}}},
                    "executionMetadata": {"tags": [{"key": "fruit", "value": "apple"}]},
                }
            },
        )
        run_id_one = payload_one["run"]["runId"]

        read_context = graphql_context

        result = execute_dagster_graphql(read_context, RUNS_QUERY, variables={"selector": selector})

        runs = result.data["pipelineOrError"]["runs"]
        assert len(runs) == 1

        tags = runs[0]["tags"]
        assert len(tags) == 1

        assert tags[0]["key"] == "fruit"
        assert tags[0]["value"] == "apple"

        payload_two = sync_execute_get_run_log_data(
            context=graphql_context,
            variables={
                "executionParams": {
                    "selector": selector,
                    "mode": "add_mode",
                    "runConfigData": {"resources": {"op": {"config": 3}}},
                    "executionMetadata": {"tags": [{"key": "veggie", "value": "carrot"}]},
                }
            },
        )

        run_id_two = payload_two["run"]["runId"]

        result = execute_dagster_graphql(read_context, RUNS_QUERY, variables={"selector": selector})

        runs = result.data["pipelineOrError"]["runs"]
        assert len(runs) == 2

        all_tags_result = execute_dagster_graphql(read_context, ALL_TAGS_QUERY)
        tags = all_tags_result.data["pipelineRunTags"]

        assert len(tags) == 2
        tags_dict = {item["key"]: item["values"] for item in tags}

        assert tags_dict == {
            "fruit": ["apple"],
            "veggie": ["carrot"],
        }

        # delete the second run
        result = execute_dagster_graphql(
            read_context, DELETE_RUN_MUTATION, variables={"runId": run_id_two}
        )
        assert result.data["deletePipelineRun"]["__typename"] == "DeletePipelineRunSuccess"
        assert result.data["deletePipelineRun"]["runId"] == run_id_two

        # query it back out
        result = execute_dagster_graphql(read_context, RUNS_QUERY, variables={"selector": selector})

        # first is the same
        run_one_data = _get_runs_data(result, run_id_one)
        assert run_one_data

        # second is gone
        run_two_data = _get_runs_data(result, run_id_two)
        assert run_two_data is None

        # try to delete the second run again
        execute_dagster_graphql(read_context, DELETE_RUN_MUTATION, variables={"runId": run_id_two})

        result = execute_dagster_graphql(
            read_context, DELETE_RUN_MUTATION, variables={"runId": run_id_two}
        )
        assert result.data["deletePipelineRun"]["__typename"] == "PipelineRunNotFoundError"


def get_repo_at_time_1():
    @lambda_solid
    def solid_A():
        pass

    @lambda_solid
    def solid_B():
        pass

    @pipeline
    def evolving_pipeline():
        solid_A()
        solid_B()

    @pipeline
    def foo_pipeline():
        solid_A()

    @repository
    def evolving_repo():
        return [evolving_pipeline, foo_pipeline]

    return evolving_repo


def get_repo_at_time_2():
    @lambda_solid
    def solid_A():
        pass

    @lambda_solid
    def solid_B_prime():
        pass

    @pipeline
    def evolving_pipeline():
        solid_A()
        solid_B_prime()

    @pipeline
    def bar_pipeline():
        solid_A()

    @repository
    def evolving_repo():
        return [evolving_pipeline, bar_pipeline]

    return evolving_repo


def test_runs_over_time():
    with seven.TemporaryDirectory() as temp_dir:
        instance = DagsterInstance.local_temp(temp_dir)

        repo_1 = get_repo_at_time_1()

        full_evolve_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline"), instance=instance
        ).run_id
        foo_run_id = execute_pipeline(repo_1.get_pipeline("foo_pipeline"), instance=instance).run_id
        evolve_a_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline").get_pipeline_subset_def({"solid_A"}),
            instance=instance,
        ).run_id
        evolve_b_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline").get_pipeline_subset_def({"solid_B"}),
            instance=instance,
        ).run_id

        context_at_time_1 = define_out_of_process_context(__file__, "get_repo_at_time_1", instance)

        result = execute_dagster_graphql(context_at_time_1, ALL_RUNS_QUERY)
        assert result.data

        t1_runs = {run["runId"]: run for run in result.data["pipelineRunsOrError"]["results"]}

        assert t1_runs[full_evolve_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": None,
        }

        assert t1_runs[foo_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "foo_pipeline",
            "solidSelection": None,
        }

        assert t1_runs[evolve_a_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": ["solid_A"],
        }

        assert t1_runs[evolve_b_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": ["solid_B"],
        }

        context_at_time_2 = define_out_of_process_context(__file__, "get_repo_at_time_2", instance)

        result = execute_dagster_graphql(context_at_time_2, ALL_RUNS_QUERY)
        assert result.data

        t2_runs = {run["runId"]: run for run in result.data["pipelineRunsOrError"]["results"]}

        assert t2_runs[full_evolve_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": None,
        }

        assert t2_runs[evolve_a_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": ["solid_A"],
        }
        # pipeline name changed
        assert t2_runs[foo_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "foo_pipeline",
            "solidSelection": None,
        }
        # subset no longer valid - b renamed
        assert t2_runs[evolve_b_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": ["solid_B"],
        }


def test_run_groups_over_time():
    with seven.TemporaryDirectory() as tempdir:
        instance = DagsterInstance.local_temp(tempdir=tempdir)

        repo_1 = get_repo_at_time_1()

        full_evolve_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline"), instance=instance
        ).run_id
        foo_run_id = execute_pipeline(repo_1.get_pipeline("foo_pipeline"), instance=instance).run_id
        evolve_a_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline").get_pipeline_subset_def({"solid_A"}),
            instance=instance,
        ).run_id
        evolve_b_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline").get_pipeline_subset_def({"solid_B"}),
            instance=instance,
        ).run_id

        context_at_time_1 = define_out_of_process_context(__file__, "get_repo_at_time_1", instance)

        result = execute_dagster_graphql(context_at_time_1, ALL_RUN_GROUPS_QUERY)
        assert result.data
        assert "runGroupsOrError" in result.data
        assert "results" in result.data["runGroupsOrError"]
        assert len(result.data["runGroupsOrError"]["results"]) == 4

        t1_runs = {
            run["runId"]: run
            for group in result.data["runGroupsOrError"]["results"]
            for run in group["runs"]
        }

        # test full_evolve_run_id
        assert t1_runs[full_evolve_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": None,
        }

        # test foo_run_id
        assert t1_runs[foo_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "foo_pipeline",
            "solidSelection": None,
        }

        # test evolve_a_run_id
        assert t1_runs[evolve_a_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": ["solid_A"],
        }
        assert t1_runs[evolve_a_run_id]["pipelineSnapshotId"]

        # test evolve_b_run_id
        assert t1_runs[evolve_b_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": ["solid_B"],
        }

        context_at_time_2 = define_out_of_process_context(__file__, "get_repo_at_time_2", instance)

        result = execute_dagster_graphql(context_at_time_2, ALL_RUN_GROUPS_QUERY)
        assert "runGroupsOrError" in result.data
        assert "results" in result.data["runGroupsOrError"]
        assert len(result.data["runGroupsOrError"]["results"]) == 4

        t2_runs = {
            run["runId"]: run
            for group in result.data["runGroupsOrError"]["results"]
            for run in group["runs"]
        }

        # test full_evolve_run_id
        assert t2_runs[full_evolve_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": None,
        }

        # test evolve_a_run_id
        assert t2_runs[evolve_a_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": ["solid_A"],
        }
        assert t2_runs[evolve_a_run_id]["pipelineSnapshotId"]

        # names same
        assert (
            t1_runs[full_evolve_run_id]["pipeline"]["name"]
            == t2_runs[evolve_a_run_id]["pipeline"]["name"]
        )

        # snapshots differ
        assert (
            t1_runs[full_evolve_run_id]["pipelineSnapshotId"]
            != t2_runs[evolve_a_run_id]["pipelineSnapshotId"]
        )

        # pipeline name changed
        assert t2_runs[foo_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "foo_pipeline",
            "solidSelection": None,
        }
        # subset no longer valid - b renamed
        assert t2_runs[evolve_b_run_id]["pipeline"] == {
            "__typename": "PipelineSnapshot",
            "name": "evolving_pipeline",
            "solidSelection": ["solid_B"],
        }


def test_filtered_runs():
    with seven.TemporaryDirectory() as temp_dir:
        instance = DagsterInstance.local_temp(temp_dir)
        repo = get_repo_at_time_1()
        run_id_1 = execute_pipeline(
            repo.get_pipeline("foo_pipeline"), instance=instance, tags={"run": "one"}
        ).run_id
        _run_id_2 = execute_pipeline(
            repo.get_pipeline("foo_pipeline"), instance=instance, tags={"run": "two"}
        ).run_id
        context = define_out_of_process_context(__file__, "get_repo_at_time_1", instance)
        result = execute_dagster_graphql(
            context, FILTERED_RUN_QUERY, variables={"filter": {"runId": run_id_1}}
        )
        assert result.data
        run_ids = [run["runId"] for run in result.data["pipelineRunsOrError"]["results"]]
        assert len(run_ids) == 1
        assert run_ids[0] == run_id_1

        result = execute_dagster_graphql(
            context,
            FILTERED_RUN_QUERY,
            variables={"filter": {"tags": [{"key": "run", "value": "one"}]}},
        )
        assert result.data
        run_ids = [run["runId"] for run in result.data["pipelineRunsOrError"]["results"]]
        assert len(run_ids) == 1
        assert run_ids[0] == run_id_1


def test_run_group():
    with seven.TemporaryDirectory() as temp_dir:
        instance = DagsterInstance.local_temp(temp_dir)
        repo = get_repo_at_time_1()
        foo_pipeline = repo.get_pipeline("foo_pipeline")
        runs = [execute_pipeline(foo_pipeline, instance=instance)]
        root_run_id = runs[-1].run_id
        for _ in range(3):
            # https://github.com/dagster-io/dagster/issues/2433
            run = instance.create_run_for_pipeline(
                foo_pipeline,
                parent_run_id=root_run_id,
                root_run_id=root_run_id,
                tags={PARENT_RUN_ID_TAG: root_run_id, ROOT_RUN_ID_TAG: root_run_id},
            )
            execute_run(InMemoryPipeline(foo_pipeline), run, instance)
            runs.append(run)

        context_at_time_1 = define_out_of_process_context(__file__, "get_repo_at_time_1", instance)

        result_one = execute_dagster_graphql(
            context_at_time_1, RUN_GROUP_QUERY, variables={"runId": root_run_id},
        )
        assert result_one.data["runGroupOrError"]["__typename"] == "RunGroup"

        assert len(result_one.data["runGroupOrError"]["runs"]) == 4

        result_two = execute_dagster_graphql(
            context_at_time_1, RUN_GROUP_QUERY, variables={"runId": runs[-1].run_id},
        )
        assert result_one.data["runGroupOrError"]["__typename"] == "RunGroup"
        assert len(result_two.data["runGroupOrError"]["runs"]) == 4

        assert (
            result_one.data["runGroupOrError"]["rootRunId"]
            == result_two.data["runGroupOrError"]["rootRunId"]
        )
        assert (
            result_one.data["runGroupOrError"]["runs"] == result_two.data["runGroupOrError"]["runs"]
        )


def test_run_group_not_found():
    with seven.TemporaryDirectory() as temp_dir:
        instance = DagsterInstance.local_temp(temp_dir)
        context_at_time_1 = define_out_of_process_context(__file__, "get_repo_at_time_1", instance)

        result = execute_dagster_graphql(
            context_at_time_1, RUN_GROUP_QUERY, variables={"runId": "foo"},
        )
        assert result.data
        assert result.data["runGroupOrError"]
        assert result.data["runGroupOrError"]["__typename"] == "RunGroupNotFoundError"
        assert result.data["runGroupOrError"]["runId"] == "foo"
        assert result.data["runGroupOrError"][
            "message"
        ] == "Run group of run {run_id} could not be found.".format(run_id="foo")


def test_run_groups():
    with seven.TemporaryDirectory() as tempdir:
        instance = DagsterInstance.local_temp(tempdir=tempdir)

        repo = get_repo_at_time_1()
        foo_pipeline = repo.get_pipeline("foo_pipeline")

        root_run_ids = [execute_pipeline(foo_pipeline, instance=instance).run_id for i in range(3)]

        for _ in range(5):
            for root_run_id in root_run_ids:
                execute_pipeline(
                    foo_pipeline,
                    tags={PARENT_RUN_ID_TAG: root_run_id, ROOT_RUN_ID_TAG: root_run_id},
                    instance=instance,
                )

        context_at_time_1 = define_out_of_process_context(__file__, "get_repo_at_time_1", instance)
        result = execute_dagster_graphql(context_at_time_1, ALL_RUN_GROUPS_QUERY)

        assert result.data
        assert "runGroupsOrError" in result.data
        assert "results" in result.data["runGroupsOrError"]
        assert len(result.data["runGroupsOrError"]["results"]) == 3
        for run_group in result.data["runGroupsOrError"]["results"]:
            assert run_group["rootRunId"] in root_run_ids
            assert len(run_group["runs"]) == 6



class PipelineRunObservableSubscribe(object):
    def __init__(self, instance, run_id, after_cursor=None):
        self.instance = instance
        self.run_id = run_id
        self.observer = None
        self.after_cursor = after_cursor if after_cursor is not None else -1

    def __call__(self, observer):
        self.observer = observer

        events = self.instance.logs_after(self.run_id, self.after_cursor)
        if events:
            self.observer.on_next(events)

        cursor = len(events) + int(self.after_cursor)
        self.instance.watch_event_logs(self.run_id, cursor, self.handle_new_event)

    def handle_new_event(self, new_event):
        self.observer.on_next([new_event])



from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.core.events import DagsterEventType
from dagster.core.host_representation import ExternalExecutionPlan, ExternalPipeline
from dagster.core.instance import DagsterInstance
from dagster.core.storage.tags import RESUME_RETRY_TAG

from .external import get_external_execution_plan_or_raise
from .utils import ExecutionParams


def get_retry_steps_from_execution_plan(instance, execution_plan, parent_run_id):
    check.inst_param(instance, "instance", DagsterInstance)
    check.inst_param(execution_plan, "execution_plan", ExternalExecutionPlan)
    check.opt_str_param(parent_run_id, "parent_run_id")

    if not parent_run_id:
        return execution_plan.step_keys_in_plan

    parent_run = instance.get_run_by_id(parent_run_id)
    parent_run_logs = instance.all_logs(parent_run_id)
    all_steps_in_parent_run_logs = set([])
    failed_steps_in_parent_run_logs = set([])
    successful_steps_in_parent_run_logs = set([])
    interrupted_steps_in_parent_run_logs = set([])
    skipped_steps_in_parent_run_logs = set([])

    for record in parent_run_logs:
        if record.dagster_event and record.dagster_event.step_key:
            all_steps_in_parent_run_logs.add(record.dagster_event.step_key)
            if record.dagster_event_type == DagsterEventType.STEP_FAILURE:
                failed_steps_in_parent_run_logs.add(record.dagster_event.step_key)
            if record.dagster_event_type == DagsterEventType.STEP_SUCCESS:
                successful_steps_in_parent_run_logs.add(record.dagster_event.step_key)
            if record.dagster_event_type == DagsterEventType.STEP_SKIPPED:
                skipped_steps_in_parent_run_logs.add(record.dagster_event.step_key)

    for step_key in all_steps_in_parent_run_logs:
        if (
            step_key not in failed_steps_in_parent_run_logs
            and step_key not in successful_steps_in_parent_run_logs
            and step_key not in skipped_steps_in_parent_run_logs
        ):
            interrupted_steps_in_parent_run_logs.add(step_key)

    to_retry = []

    execution_deps = execution_plan.execution_deps()
    for step in execution_plan.topological_steps():
        if parent_run.step_keys_to_execute and step.key not in parent_run.step_keys_to_execute:
            continue

        if (
            step.key in failed_steps_in_parent_run_logs
            # Interrupted steps can occur when graceful cleanup from a step failure fails to run,
            # and a step failure event is not generated
            or step.key in interrupted_steps_in_parent_run_logs
            # Missing steps did not execute, e.g. when a run was terminated
            or step.key not in all_steps_in_parent_run_logs
        ):
            to_retry.append(step.key)
            continue

        step_deps = execution_deps[step.key]
        if step_deps.intersection(to_retry):
            # this step is downstream of a step we are about to retry
            to_retry.append(step.key)

    return to_retry


def compute_step_keys_to_execute(graphene_info, external_pipeline, execution_params):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
    check.inst_param(execution_params, "execution_params", ExecutionParams)

    instance = graphene_info.context.instance

    if not execution_params.step_keys and is_resume_retry(execution_params):
        # Get step keys from parent_run_id if it's a resume/retry
        external_execution_plan = get_external_execution_plan_or_raise(
            graphene_info=graphene_info,
            external_pipeline=external_pipeline,
            mode=execution_params.mode,
            run_config=execution_params.run_config,
            step_keys_to_execute=None,
        )
        return get_retry_steps_from_execution_plan(
            instance, external_execution_plan, execution_params.execution_metadata.parent_run_id
        )
    else:
        return execution_params.step_keys


def is_resume_retry(execution_params):
    check.inst_param(execution_params, "execution_params", ExecutionParams)
    return execution_params.execution_metadata.tags.get(RESUME_RETRY_TAG) == "true"



from dagster_graphql.client.query import LAUNCH_TRIGGERED_EXECUTION
from dagster_graphql.test.utils import execute_dagster_graphql, infer_repository_selector

from .graphql_context_test_suite import OutOfProcessExecutingGraphQLContextTestMatrix


class TestTriggerRuns(OutOfProcessExecutingGraphQLContextTestMatrix):
    def test_get_partition_runs(self, graphql_context):
        repository_selector = infer_repository_selector(graphql_context)
        result = execute_dagster_graphql(
            graphql_context,
            LAUNCH_TRIGGERED_EXECUTION,
            variables={
                "triggerSelector": {
                    "repositoryName": repository_selector["repositoryName"],
                    "repositoryLocationName": repository_selector["repositoryLocationName"],
                    "jobName": "job_no_config",
                }
            },
        )
        assert not result.errors
        assert result.data["triggerExecution"]["__typename"] == "TriggerExecutionSuccess"
        assert len(result.data["triggerExecution"]["launchedRunIds"]) == 1



from .utils import execute_dagster_graphql, infer_pipeline_selector

PRESETS_QUERY = """
query PresetsQuery($selector: PipelineSelector!) {
  pipelineOrError(params: $selector) {
    ... on Pipeline {
      name
      presets {
        __typename
        name
        solidSelection
        runConfigYaml
        mode
      }
    }
  }
}
"""


def execute_preset_query(pipeline_name, context):
    selector = infer_pipeline_selector(context, pipeline_name)
    return execute_dagster_graphql(context, PRESETS_QUERY, variables={"selector": selector})



import sys

from dagster_graphql.test.utils import execute_dagster_graphql

from dagster import repository
from dagster.core.code_pointer import CodePointer
from dagster.core.host_representation import (
    ExternalRepository,
    RepositoryHandle,
    external_repository_data_from_def,
)
from dagster.grpc.types import ListRepositoriesResponse
from dagster.seven import mock

from .graphql_context_test_suite import GraphQLContextVariant, make_graphql_context_test_suite

# import dagster.api.utils


class TestReloadRepositoriesOutOfProcess(
    make_graphql_context_test_suite(
        context_variants=[GraphQLContextVariant.readonly_in_memory_instance_out_of_process_env()]
    )
):
    def test_out_of_process_reload_location(self, graphql_context):
        result = execute_dagster_graphql(
            graphql_context, RELOAD_REPOSITORY_LOCATION_QUERY, {"repositoryLocationName": "test"}
        )

        assert result
        assert result.data
        assert result.data["reloadRepositoryLocation"]
        assert result.data["reloadRepositoryLocation"]["__typename"] == "RepositoryLocation"
        assert result.data["reloadRepositoryLocation"]["name"] == "test"
        assert result.data["reloadRepositoryLocation"]["repositories"] == [{"name": "test_repo"}]
        assert result.data["reloadRepositoryLocation"]["isReloadSupported"] is True

        with mock.patch(
            # note it where the function is *used* that needs to mocked, not
            # where it is defined.
            # see https://docs.python.org/3/library/unittest.mock.html#where-to-patch
            "dagster.core.host_representation.handle.sync_list_repositories_grpc"
        ) as cli_command_mock:

            with mock.patch(
                # note it where the function is *used* that needs to mocked, not
                # where it is defined.
                # see https://docs.python.org/3/library/unittest.mock.html#where-to-patch
                "dagster.core.host_representation.repository_location.sync_get_streaming_external_repositories_grpc"
            ) as external_repository_mock:

                @repository
                def new_repo():
                    return []

                new_repo_data = external_repository_data_from_def(new_repo)

                external_repository_mock.return_value = [
                    ExternalRepository(
                        new_repo_data,
                        RepositoryHandle(
                            "new_repo", graphql_context.repository_locations[0].location_handle
                        ),
                    )
                ]

                cli_command_mock.return_value = ListRepositoriesResponse(
                    repository_symbols=[],
                    executable_path=sys.executable,
                    repository_code_pointer_dict={
                        "new_repo": CodePointer.from_python_file(__file__, "new_repo", None)
                    },
                )

                result = execute_dagster_graphql(
                    graphql_context,
                    RELOAD_REPOSITORY_LOCATION_QUERY,
                    {"repositoryLocationName": "test"},
                )

                assert cli_command_mock.call_count == 1
                assert external_repository_mock.call_count == 1

                assert result.data["reloadRepositoryLocation"]["repositories"] == [
                    {"name": "new_repo"}
                ]


class TestReloadRepositoriesInProcess(
    make_graphql_context_test_suite(
        context_variants=[GraphQLContextVariant.readonly_in_memory_instance_in_process_env()]
    )
):
    def test_reload_not_supported(self, graphql_context):
        result = execute_dagster_graphql(
            graphql_context,
            RELOAD_REPOSITORY_LOCATION_QUERY,
            {"repositoryLocationName": "<<in_process>>"},
        )
        assert result.data["reloadRepositoryLocation"]["__typename"] == "ReloadNotSupported"

    def test_location_not_found(self, graphql_context):
        result = execute_dagster_graphql(
            graphql_context, RELOAD_REPOSITORY_LOCATION_QUERY, {"repositoryLocationName": "nope"},
        )
        assert result.data["reloadRepositoryLocation"]["__typename"] == "RepositoryLocationNotFound"


RELOAD_REPOSITORY_LOCATION_QUERY = """
mutation ($repositoryLocationName: String!) {
   reloadRepositoryLocation(repositoryLocationName: $repositoryLocationName) {
      __typename
      ... on RepositoryLocation {
        name
        repositories {
            name
        }
        isReloadSupported
      }
   }
}
"""


class TestReloadRepositoriesManagedGrpc(
    make_graphql_context_test_suite(
        context_variants=[GraphQLContextVariant.readonly_in_memory_instance_managed_grpc_env(),]
    )
):
    def test_managed_grpc_reload_location(self, graphql_context):
        result = execute_dagster_graphql(
            graphql_context, RELOAD_REPOSITORY_LOCATION_QUERY, {"repositoryLocationName": "test"}
        )

        assert result
        assert result.data
        assert result.data["reloadRepositoryLocation"]
        assert result.data["reloadRepositoryLocation"]["__typename"] == "RepositoryLocation"
        assert result.data["reloadRepositoryLocation"]["name"] == "test"
        assert result.data["reloadRepositoryLocation"]["repositories"] == [{"name": "test_repo"}]
        assert result.data["reloadRepositoryLocation"]["isReloadSupported"] is True



from dagster.core.storage.pipeline_run import PipelineRunStatus
from dagster.core.utils import make_new_run_id
from dagster.utils import merge_dicts

from ..external import ensure_valid_config, get_external_execution_plan_or_raise
from ..resume_retry import compute_step_keys_to_execute


def create_valid_pipeline_run(graphene_info, external_pipeline, execution_params):
    ensure_valid_config(external_pipeline, execution_params.mode, execution_params.run_config)

    step_keys_to_execute = compute_step_keys_to_execute(
        graphene_info, external_pipeline, execution_params
    )

    external_execution_plan = get_external_execution_plan_or_raise(
        graphene_info=graphene_info,
        external_pipeline=external_pipeline,
        mode=execution_params.mode,
        run_config=execution_params.run_config,
        step_keys_to_execute=step_keys_to_execute,
    )

    return graphene_info.context.instance.create_run(
        pipeline_snapshot=external_pipeline.pipeline_snapshot,
        execution_plan_snapshot=external_execution_plan.execution_plan_snapshot,
        parent_pipeline_snapshot=external_pipeline.parent_pipeline_snapshot,
        pipeline_name=execution_params.selector.pipeline_name,
        run_id=execution_params.execution_metadata.run_id
        if execution_params.execution_metadata.run_id
        else make_new_run_id(),
        solids_to_execute=frozenset(execution_params.selector.solid_selection)
        if execution_params.selector.solid_selection
        else None,
        run_config=execution_params.run_config,
        mode=execution_params.mode,
        step_keys_to_execute=step_keys_to_execute,
        tags=merge_dicts(external_pipeline.tags, execution_params.execution_metadata.tags),
        root_run_id=execution_params.execution_metadata.root_run_id,
        parent_run_id=execution_params.execution_metadata.parent_run_id,
        status=PipelineRunStatus.NOT_STARTED,
    )



from dagster_graphql.implementation.external import (
    get_external_pipeline_or_raise,
    get_full_external_pipeline_or_raise,
)
from dagster_graphql.implementation.utils import PipelineSelector
from dagster_graphql.schema.pipelines import DauphinPipeline, DauphinPipelineSnapshot
from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.core.storage.pipeline_run import PipelineRun

from .utils import UserFacingGraphQLError, capture_dauphin_error


@capture_dauphin_error
def get_pipeline_snapshot_or_error_from_pipeline_selector(graphene_info, pipeline_selector):
    check.inst_param(pipeline_selector, "pipeline_selector", PipelineSelector)
    return DauphinPipelineSnapshot(
        get_full_external_pipeline_or_raise(graphene_info, pipeline_selector)
    )


@capture_dauphin_error
def get_pipeline_snapshot_or_error_from_snapshot_id(graphene_info, snapshot_id):
    check.str_param(snapshot_id, "snapshot_id")
    return _get_dauphin_pipeline_snapshot_from_instance(graphene_info.context.instance, snapshot_id)


# extracted this out to test
def _get_dauphin_pipeline_snapshot_from_instance(instance, snapshot_id):
    from dagster_graphql.schema.errors import DauphinPipelineSnapshotNotFoundError

    if not instance.has_pipeline_snapshot(snapshot_id):
        raise UserFacingGraphQLError(DauphinPipelineSnapshotNotFoundError(snapshot_id))

    historical_pipeline = instance.get_historical_pipeline(snapshot_id)

    if not historical_pipeline:
        # Either a temporary error or it has been deleted in the interim
        raise UserFacingGraphQLError(DauphinPipelineSnapshotNotFoundError(snapshot_id))

    return DauphinPipelineSnapshot(historical_pipeline)


@capture_dauphin_error
def get_pipeline_or_error(graphene_info, selector):
    """Returns a DauphinPipelineOrError."""
    return get_dauphin_pipeline_from_selector(graphene_info, selector)


def get_pipeline_or_raise(graphene_info, selector):
    """Returns a DauphinPipeline or raises a UserFacingGraphQLError if one cannot be retrieved
    from the selector, e.g., the pipeline is not present in the loaded repository."""
    return get_dauphin_pipeline_from_selector(graphene_info, selector)


def get_pipeline_reference_or_raise(graphene_info, pipeline_run):
    """Returns a DauphinPipelineReference or raises a UserFacingGraphQLError if a pipeline
    reference cannot be retrieved based on the run, e.g, a UserFacingGraphQLError that wraps an
    InvalidSubsetError."""
    check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
    solid_selection = (
        list(pipeline_run.solids_to_execute) if pipeline_run.solids_to_execute else None
    )

    if pipeline_run.pipeline_snapshot_id is None:
        return graphene_info.schema.type_named("UnknownPipeline")(
            pipeline_run.pipeline_name, solid_selection
        )

    return _get_dauphin_pipeline_snapshot_from_instance(
        graphene_info.context.instance, pipeline_run.pipeline_snapshot_id
    )


def get_dauphin_pipeline_from_selector(graphene_info, selector):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(selector, "selector", PipelineSelector)

    return DauphinPipeline(get_external_pipeline_or_raise(graphene_info, selector))



from __future__ import absolute_import

from graphql.execution.base import ResolveInfo

from dagster import check

from ..external import get_external_pipeline_or_raise
from ..utils import ExecutionMetadata, ExecutionParams, capture_dauphin_error
from .run_lifecycle import create_valid_pipeline_run


@capture_dauphin_error
def launch_pipeline_reexecution(graphene_info, execution_params):
    return _launch_pipeline_execution(graphene_info, execution_params, is_reexecuted=True)


@capture_dauphin_error
def launch_pipeline_execution(graphene_info, execution_params):
    return _launch_pipeline_execution(graphene_info, execution_params)


def do_launch(graphene_info, execution_params, is_reexecuted=False):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(execution_params, "execution_params", ExecutionParams)
    check.bool_param(is_reexecuted, "is_reexecuted")

    if is_reexecuted:
        # required fields for re-execution
        execution_metadata = check.inst_param(
            execution_params.execution_metadata, "execution_metadata", ExecutionMetadata
        )
        check.str_param(execution_metadata.root_run_id, "root_run_id")
        check.str_param(execution_metadata.parent_run_id, "parent_run_id")

    external_pipeline = get_external_pipeline_or_raise(graphene_info, execution_params.selector)

    pipeline_run = create_valid_pipeline_run(graphene_info, external_pipeline, execution_params)

    return graphene_info.context.instance.launch_run(
        pipeline_run.run_id, external_pipeline=external_pipeline
    )


def _launch_pipeline_execution(graphene_info, execution_params, is_reexecuted=False):
    check.inst_param(graphene_info, "graphene_info", ResolveInfo)
    check.inst_param(execution_params, "execution_params", ExecutionParams)
    check.bool_param(is_reexecuted, "is_reexecuted")

    run = do_launch(graphene_info, execution_params, is_reexecuted)

    return graphene_info.schema.type_named("LaunchPipelineRunSuccess")(
        run=graphene_info.schema.type_named("PipelineRun")(run)
    )



from dagster_graphql import dauphin
from dagster_graphql.implementation.fetch_schedules import (
    reconcile_scheduler_state,
    start_schedule,
    stop_schedule,
)
from dagster_graphql.schema.errors import (
    DauphinPythonError,
    DauphinRepositoryNotFoundError,
    DauphinScheduleDefinitionNotFoundError,
    DauphinScheduleNotFoundError,
    DauphinSchedulerNotDefinedError,
)

from dagster import check
from dagster.core.host_representation import ExternalSchedule, ScheduleSelector
from dagster.core.host_representation.selector import RepositorySelector
from dagster.core.scheduler import ScheduleState, ScheduleTickStatus
from dagster.core.scheduler.scheduler import ScheduleTickStatsSnapshot
from dagster.core.storage.pipeline_run import PipelineRunsFilter

from .schedule_definition import (
    DapuphinScheduleDefinitionOrError,
    DauphinScheduleDefinition,
    DauphinScheduleDefinitions,
    DauphinScheduleDefintionsOrError,
)
from .schedule_state import (
    DapuphinScheduleStateOrError,
    DauphinScheduleState,
    DauphinScheduleStates,
    DauphinScheduleStatesOrError,
)


class DauphinScheduleStatus(dauphin.Enum):
    class Meta(object):
        name = "ScheduleStatus"

    RUNNING = "RUNNING"
    STOPPED = "STOPPED"
    ENDED = "ENDED"


class DauphinSchedulerOrError(dauphin.Union):
    class Meta(object):
        name = "SchedulerOrError"
        types = ("Scheduler", DauphinSchedulerNotDefinedError, "PythonError")


DauphinScheduleTickStatus = dauphin.Enum.from_enum(ScheduleTickStatus)


class DauphinScheduleTick(dauphin.ObjectType):
    class Meta(object):
        name = "ScheduleTick"

    tick_id = dauphin.NonNull(dauphin.String)
    status = dauphin.NonNull("ScheduleTickStatus")
    timestamp = dauphin.NonNull(dauphin.Float)
    tick_specific_data = dauphin.Field("ScheduleTickSpecificData")


class DauphinScheduleTickSuccessData(dauphin.ObjectType):
    class Meta(object):
        name = "ScheduleTickSuccessData"

    run = dauphin.Field("PipelineRun")


class DauphinScheduleTickFailureData(dauphin.ObjectType):
    class Meta(object):
        name = "ScheduleTickFailureData"

    error = dauphin.NonNull("PythonError")


class DauphinScheduleTickSpecificData(dauphin.Union):
    class Meta(object):
        name = "ScheduleTickSpecificData"
        types = (
            DauphinScheduleTickSuccessData,
            DauphinScheduleTickFailureData,
        )


class DauphinScheduleTickStatsSnapshot(dauphin.ObjectType):
    class Meta(object):
        name = "ScheduleTickStatsSnapshot"

    ticks_started = dauphin.NonNull(dauphin.Int)
    ticks_succeeded = dauphin.NonNull(dauphin.Int)
    ticks_skipped = dauphin.NonNull(dauphin.Int)
    ticks_failed = dauphin.NonNull(dauphin.Int)

    def __init__(self, stats):
        super(DauphinScheduleTickStatsSnapshot, self).__init__(
            ticks_started=stats.ticks_started,
            ticks_succeeded=stats.ticks_succeeded,
            ticks_skipped=stats.ticks_skipped,
            ticks_failed=stats.ticks_failed,
        )
        self._stats = check.inst_param(stats, "stats", ScheduleTickStatsSnapshot)


class DauphinScheduler(dauphin.ObjectType):
    class Meta(object):
        name = "Scheduler"

    scheduler_class = dauphin.String()


class DauphinReconcileSchedulerStateSuccess(dauphin.ObjectType):
    class Meta(object):
        name = "ReconcileSchedulerStateSuccess"

    message = dauphin.NonNull(dauphin.String)


class DauphinReconcilScheduleStateMutationResult(dauphin.Union):
    class Meta(object):
        name = "ReconcileSchedulerStateMutationResult"
        types = (DauphinPythonError, DauphinReconcileSchedulerStateSuccess)


class DauphinReconcileSchedulerStateMutation(dauphin.Mutation):
    class Meta(object):
        name = "ReconcileSchedulerStateMutation"

    class Arguments(object):
        repository_selector = dauphin.NonNull("RepositorySelector")

    Output = dauphin.NonNull("ReconcileSchedulerStateMutationResult")

    def mutate(self, graphene_info, repository_selector):
        return reconcile_scheduler_state(
            graphene_info, RepositorySelector.from_graphql_input(repository_selector)
        )


class DauphinScheduleStateResult(dauphin.ObjectType):
    class Meta(object):
        name = "ScheduleStateResult"

    schedule_state = dauphin.NonNull("ScheduleState")


class DauphinScheduleMutationResult(dauphin.Union):
    class Meta(object):
        name = "ScheduleMutationResult"
        types = (DauphinPythonError, DauphinScheduleStateResult)


class DauphinStartScheduleMutation(dauphin.Mutation):
    class Meta(object):
        name = "StartScheduleMutation"

    class Arguments(object):
        schedule_selector = dauphin.NonNull("ScheduleSelector")

    Output = dauphin.NonNull("ScheduleMutationResult")

    def mutate(self, graphene_info, schedule_selector):
        return start_schedule(graphene_info, ScheduleSelector.from_graphql_input(schedule_selector))


class DauphinStopRunningScheduleMutation(dauphin.Mutation):
    class Meta(object):
        name = "StopRunningScheduleMutation"

    class Arguments(object):
        schedule_origin_id = dauphin.NonNull(dauphin.String)

    Output = dauphin.NonNull("ScheduleMutationResult")

    def mutate(self, graphene_info, schedule_origin_id):
        return stop_schedule(graphene_info, schedule_origin_id)
